export enum USER_TYPE {
  ORGANIZATION = "ORGANIZATION",
  CANDIDATE = "CANDIDATE",
}

export enum APP_STATE {
  LOADING = "loading",
  FAILED = "failed",
  SUCCESS = "success",
}

export enum OnboardSteps {
  RESUME = "RESUME",
  SELF_INTRO = "SELF_INTRO",
  COMPLETE = "COMPLETE",
}

export type KeyDetails = {
  key: any;
  code: any;
  shiftKey: any;
  time: number | null;
};

export type Aspects = {
  dd: number | boolean;
  flight: number | boolean;
  full: number | boolean;
  hold: number | boolean;
};

export type Stat = {
  standard: {
    distance: Aspects;
    inRange: Aspects;
    inRangePercent: Aspects;
    threshold: number;
    sd: number;
  };
  filtered: {
    distance: Aspects;
    inRange: Aspects;
    inRangePercent: Aspects;
    threshold: number;
    sd: number;
  };
  mahalanobis: {
    threshold: number;
    normedDistance: Aspects;
    distance: Aspects;
    inRange: Aspects;
  };
  fullStandard: {
    threshold: number;
    normedDistance: Aspects;
    distance: Aspects;
    inRange: Aspects;
  };
  fullFiltered: {
    threshold: number;
    normedDistance: Aspects;
    distance: Aspects;
    inRange: Aspects;
  };
  accepted: boolean;
};

export type KeystrokeType = {
  hold: number[];
  flight: number[];
  dd: number[];
  full: number[];
};

export type KeystrokeResultType = {
  attempt: KeystrokeType;
  db: KeystrokeType;
  filteredDb: KeystrokeType;
  result: Stat | null;
  controls: ControlsType;
};

export type AddressType = {
  addressLine: string;
  city: string;
  country: string;
};

export type CandidateType = {
  _id?: string;
  name: string;
  bio: string;
  contacts: {
    email: string;
    phone: string;
    address: AddressType;
    residentialAddress?: AddressType;
  };
  dateOfBirth: string;
  jobIds: string[];
  profilePicture: string;
  state: "INTIAL" | "READY";
  resume?: string;
  selfIntro?: string;
};

export type OrganizationType = {
  _id?: string;
  name: string;
  description: string;
  contacts: {
    email: string;
    phone: string;
    address: AddressType;
    website: string;
  };
  profilePicture: string;
};

export type ControlsType = {
  standard: {
    sd: number;
    threshold: number;
    use: boolean;
  };
  fullStandard: {
    threshold: number;
    use: boolean;
  };
};

export type JobType = {
  _id?: string;
  title: string;
  description: string;
  primarySkills: string[];
  secondarySkills: string[];
  salary: {
    min: number;
    max: number;
    currency: string;
  };
  applications?: string[];
  organization: string;
};

export type ApplicationType = {
  candidate: string;
  job: string;
  status: "Pending" | "Accepted" | "In progress" | "Rejected";
  interview?: {
    date: string;
    time: string;
    link: string;
    videoRef?: string;
    voiceVerification?: string;
  };
};

export type EmotionsPayloadType = {
  angry: number;
  disgust: number;
  fear: number;
  happy: number;
  sad: number;
  surprise: number;
  neutral: number;
};

export type EmotionsType = {
  data: [
    {
      box: number[];
      emotions: EmotionsPayloadType;
    }
  ];
  end: boolean;
};

//PAYLOADS
export type SignUpPayload = {
  passwords: string[];
  keydown: KeyDetails[][];
  keyup: KeyDetails[][];
  email: string;
  userType: USER_TYPE;
  candidate?: CandidateType | {};
  organization?: OrganizationType | {};
};

export type SignInPayload = {
  password: string;
  keydown: KeyDetails[];
  keyup: KeyDetails[];
  email: string;
  userType: USER_TYPE;
};

export type UpdatePasswordPayload = {
  passwords: string[];
  keydown: KeyDetails[][];
  keyup: KeyDetails[][];
  oldPassword: string;
};

export type JobPayloadType = {
  _id?: string;
  title: string;
  description: string;
  primarySkills: string[];
  secondarySkills: string[];
  salary: {
    min: number;
    max: number;
    currency: string;
  };
  applications?: ApplicationType[] | ApplicationPayloadType[];
  organization: OrganizationType[];
};

export type ApplicationPayloadType = {
  _id: string;
  candidate: CandidateType;
  job: string;
  status: string;
  interview?: {
    date: string;
    time: string;
    link: string;
    videoRef?: string;
    voiceVerification?: string;
  };
  score: {
    primary: number;
    primatyMatch?: string[];
    secondary: number;
    secondaryMatch?: string[];
    similarity: number;
    total: number;
  };
};

//REDUCERS
export type AuthReducer = {
  token: string | null;
  candidate: CandidateType | {};
  organization: OrganizationType | {};
  userType: USER_TYPE;
  userId: string;
  keystrokeResult?: KeystrokeResultType;
  appState?: {
    state: APP_STATE;
    msg?: string;
  } | null;
};

export type CommonReducer = {
  jobs: JobType[] | JobPayloadType[];
  dialogAlert: string | null;
};

export type Reducers = {
  auth: AuthReducer;
  common: CommonReducer;
};
