import React, { useRef, useEffect, useState } from "react";
import { KeyDetails, UpdatePasswordPayload } from "../common/types";
import Alert from "../components/Alert";
import { keyEvent } from "../common/lib/util";
import { useDispatch } from "react-redux";
import { changePassword } from "../common/actions/auth";

const ChangePassword = () => {
  const dispatch = useDispatch();
  const keydownArray = useRef<KeyDetails[][]>([]);
  const keyupArray = useRef<KeyDetails[][]>([]);
  const keyCount = useRef<number[]>([]);
  const [alert, setAlert] = useState<string | null>(null);
  const [credentials, setCredentials] = useState({
    oldPassword: "",
    password: "",
    confrimPassword: "",
    twoFAPassword: "",
  });

  useEffect(() => {
    resetData();
  }, []);

  const resetData = () => {
    for (let i = 0; i < 3; i++) {
      keydownArray.current.push([]);
      keyupArray.current.push([]);
      keyCount.current.push(0);
    }
  };

  const onChangeCredentials = (e: React.ChangeEvent<HTMLInputElement>) => {
    setCredentials({ ...credentials, [e.target.name]: e.target.value });
  };

  const clearField = (index: number) => {
    if (index === 0) {
      setCredentials({ ...credentials, password: "" });
    } else if (index === 1) {
      setCredentials({ ...credentials, confrimPassword: "" });
    } else {
      setCredentials({ ...credentials, twoFAPassword: "" });
    }

    keydownArray.current[index] = [];
    keyupArray.current[index] = [];
    keyCount.current[index] = 0;
  };

  const onKeyDown = (e: any) => {
    const field = e.target.id;
    const index = Number(field.substr(field.length - 1));

    let details = keyEvent(e);
    if (!details) return;

    if (
      ["Backspace"].includes(details.code) ||
      ["Backspace"].includes(details.code)
    ) {
      return clearField(index);
    }

    if (
      ["Tab", "NumpadEnter"].includes(details.code) ||
      ["Tab"].includes(details.key)
    ) {
      details = {
        ...details,
        code: "Enter",
        key: "Enter",
      };
    }

    keydownArray.current[index].push(details);
    keyupArray.current[index].push({ ...details, time: null });
    keyCount.current[index]++;
  };

  const onKeyUp = (e: any) => {
    const field = e.target.id;
    const index = Number(field.substr(field.length - 1));

    let details = keyEvent(e);
    if (!details) return;
    if (!details.time) details.time = Date.now();

    if (
      ["Backspace"].includes(details.code) ||
      ["Backspace"].includes(details.code)
    ) {
      return clearField(index);
    }

    if (
      ["Tab", "NumpadEnter"].includes(details.code) ||
      ["Tab"].includes(details.key)
    ) {
      details = {
        ...details,
        code: "Enter",
        key: "Enter",
      };
    }

    let reqdUpKeystroke = keyupArray.current[index].find(
      (element) => element.code === details?.code && !element.time
    );

    if (reqdUpKeystroke) reqdUpKeystroke.time = details.time;
  };

  const onSumit = () => {
    setAlert(null);
    if (credentials.password !== credentials.confrimPassword) {
      return setAlert("Passwords do not match");
    }

    const payload: UpdatePasswordPayload = {
      oldPassword: credentials.oldPassword,
      passwords: [
        credentials.password,
        credentials.confrimPassword,
        credentials.twoFAPassword,
      ],
      keydown: keydownArray.current,
      keyup: keyupArray.current,
    };

    resetData();

    dispatch(changePassword(payload));
  };

  return (
    <div className="card p-4">
      <h5>Change password</h5>
      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">Old password</label>
        <div className="col-sm-5">
          <input
            type="password"
            className="form-control"
            name="oldPassword"
            onChange={onChangeCredentials}
            onKeyDown={onKeyDown}
            onKeyUp={onKeyUp}
          />
        </div>
      </div>
      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">New password</label>
        <div className="col-sm-5">
          <input
            type="password"
            className="form-control"
            name="password"
            onChange={onChangeCredentials}
            onKeyDown={onKeyDown}
            onKeyUp={onKeyUp}
          />
        </div>
      </div>
      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">Confirm new password</label>
        <div className="col-sm-5">
          <input
            type="password"
            className="form-control"
            name="confrimPassword"
            onChange={onChangeCredentials}
            onKeyDown={onKeyDown}
            onKeyUp={onKeyUp}
          />
        </div>
      </div>
      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">2FA confirm password</label>
        <div className="col-sm-5">
          <input
            type="password"
            className="form-control"
            name="twoFAPassword"
            onChange={onChangeCredentials}
            onKeyDown={onKeyDown}
            onKeyUp={onKeyUp}
          />
        </div>
      </div>

      <div className="d-md-block">
        <button
          type="button"
          className="btn btn-primary"
          style={{ width: "300px", float: "right" }}
          onClick={onSumit}
        >
          Save Password
        </button>
      </div>
      <Alert alert={alert} setAlert={setAlert} />
    </div>
  );
};

export default ChangePassword;
