import React, { useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { getJobs } from "../common/actions/common";
import { JobPayloadType, JobType, Reducers, USER_TYPE } from "../common/types";

type OwnProps = {
  jobs: JobType[] | JobPayloadType[];
  onSelectJob?: (item: number) => void;
  selectedIndex?: number;
};

const Jobs = ({ jobs, onSelectJob, selectedIndex }: OwnProps) => {
  const dispatch = useDispatch();
  const [isLoading, setIsLoading] = useState<boolean>(true);
  const userType = useSelector((state: Reducers) => state.auth.userType);

  useEffect(() => {
    dispatch(
      getJobs(() => {
        setIsLoading(false);
      })
    );
  }, [dispatch]);

  const renderStatus = () => {
    if (isLoading) {
      return <p style={{ textAlign: "center" }}>Loading jobs...</p>;
    } else if (jobs.length === 0) {
      if (userType === USER_TYPE.CANDIDATE) {
        return <p style={{ textAlign: "center" }}>No jobs found yet!</p>;
      } else {
        return (
          <p style={{ textAlign: "center" }}>
            You don't have posted any jobs yet!
          </p>
        );
      }
    }

    return null;
  };

  const renderJobs = (job: JobType | JobPayloadType, index: number) => {
    const onClick = () => onSelectJob?.(index);
    const className = `card p-3 job-card ${
      index === selectedIndex && "active"
    }`;
    return (
      <div className={className} key={job._id} onClick={onClick}>
        <h6>{job.title}</h6>
        <p>
          Salary : {job.salary.min} - {job.salary.max} {job.salary.currency}{" "}
        </p>
        <p className="desc">{job.description.slice(0, 200)}...</p>
        {userType === USER_TYPE.ORGANIZATION && (
          <p className="desc mt-2">
            <label>Applicattions : </label>{" "}
            <a href={`/applications?jobId=${job._id}`}>
              ({job.applications?.length}) Candidates
            </a>
          </p>
        )}
      </div>
    );
  };

  return (
    <div>
      {renderStatus()}
      {jobs.map(renderJobs)}
    </div>
  );
};

export default Jobs;
